//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 30/08/02; 20:51:59

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include "extgraph.h"
#include "memory.h"
#include "menulib.h"
#include "graphic.h"

//-----------------------------------------------------------------------------

extern TMenu * Menu;
enum {ME_MAIN,ME_PAUSE,ME_OPTIONS};

//-----------------------------------------------------------------------------

TGame *Game;

int xpos,ypos;
int pairs_size;
int pairs_counter;
int width;
int height;

int LeftLabel=103;

char*Plane0;
char*Plane1;

//-----------------------------------------------------------------------------

void NewGame(void)
{
	int tile_counter;
	int pair_size_counter;
	int x,y;
	char TileChar = 'A';
	
	
	for (x=0;x<width;x++)
	for (y=0;y<height;y++)
	Game->Tiles[x][y].State=TS_Hidden;
	
	// CREATION
	
	pairs_counter = (width*height)/pairs_size;
	
	pair_size_counter = 0;
	
	for (x=0;x<width;x++)
	for (y=0;y<height;y++)
	{
		Game->Tiles[x][y].Value = TileChar;
		
		tile_counter = width*y+x;
		
		if ((tile_counter+1)<=(pairs_counter*pairs_size))
		Game->Tiles[x][y].State = TS_Hidden;
		else
		Game->Tiles[x][y].State = TS_Used;
		
		pair_size_counter++;
		if (pair_size_counter==pairs_size)
		{
			TileChar++;
			pair_size_counter = 0;
		}
		
	}
	
	int xx,yy;
	
	// SHUFFLE
	
	for (x=0;x<width;x++)
	for (y=0;y<height;y++)
	{
		
		do
		{
			xx = random(width);
			yy = random(height);
		}
		while (!(x!=xx && y!=yy));
		
		
		TTile TileTemp=Game->Tiles[x][y];
		Game->Tiles[x][y]=Game->Tiles[xx][yy];
		Game->Tiles[xx][yy]=TileTemp;
	}
	
	xpos = 0;
	ypos = 0;
	Game->Pair.Count = 0;
	
	
	
}

//-----------------------------------------------------------------------------

void RefreshTile(int x, int y)
{
	
	int xpos=12*x;
	int ypos=12*y;
	
	
	unsigned short*ptr1=NULL;
	unsigned short*ptr2=NULL;
	
	switch (Game->Tiles[x][y].State)
	{
		case TS_Hidden :  ptr1=Image1_C0;ptr2=Image1_C1;break;
		case TS_Used :    ptr1=Image2_C0;ptr2=Image2_C1;break;
		case TS_Visible : ptr1=Image3_C0;ptr2=Image3_C1;break;
	}
	
	GraySprite16_AND(xpos,ypos,12,Image_White,Image_White,Plane0,Plane1);
	GraySprite16_OR (xpos,ypos,12,ptr1,ptr2,Plane0,Plane1);
	
	if (Game->Tiles[x][y].State!=TS_Hidden)
	DrawGrayChar2B(xpos+2,ypos+2,Game->Tiles[x][y].Value,A_NORMAL,Plane0,Plane1);
	
}

//-----------------------------------------------------------------------------

void RefreshScreen(void)
{
	int x,y;
	char buffer[256];
	
	// TILES
	
	ClearGrayScreen2B(Plane0,Plane1);
	
	for (x=0;x<width;x++)
	for (y=0;y<height;y++)
	RefreshTile(x,y);
	
	// STATS
	
	int TopLabel=0;
	
	DrawGrayStrExt2B(LeftLabel,TopLabel,"Memory",A_NORMAL,0,Plane0,Plane1);
	TopLabel+=8;
	
	sprintf(buffer,"Pairs Left : %d",pairs_counter);
	DrawGrayStrExt2B(LeftLabel,TopLabel,buffer,A_NORMAL,0,Plane0,Plane1);
	TopLabel+=8;
	
	sprintf(buffer,"Pairs Size : %d",pairs_size);
	DrawGrayStrExt2B(LeftLabel,TopLabel,buffer,A_NORMAL,0,Plane0,Plane1);
	TopLabel+=8;
	
	FastCopyScreen(Plane0,GetPlane(0));
	FastCopyScreen(Plane1,GetPlane(1));
	
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	int a,b;
	char buffer[256];
	int key;
	
	char screen_C0[LCD_SIZE];
	char screen_C1[LCD_SIZE];
	
	Plane0=screen_C0;
	Plane1=screen_C1;
	
	TGame GameA;
	Game=&GameA;
	
	TMenu MenuA;
	Menu = &MenuA;
	
	randomize();
	
	width = 6;
	height = 6;
	pairs_size = 3;
	
	FontSetSys(1);
	
	for (;;)
	{
		MenuNew2();
		MenuAddTitle("Memory");
		MenuAddItem("Play");
		MenuAddItem("Options");
		MenuAddItem("About");
		MenuAddItem("Quit");
		
		MenuDo(ME_MAIN);
		
		if (!Menu->ok) break;
		
		if (Menu->Index==1)
		{
			
			GrayOn();
			
			NewGame();
			RefreshScreen();
			
			for (;;)
			{
				
				if (pairs_counter==0)
				{
					DrawGrayStrExt2B(LeftLabel,60,"You Won!",A_NORMAL,0,GetPlane(0),GetPlane(1));
					ngetchx();
					
					NewGame();
					RefreshScreen();
				}
				
				
				GraySprite16_XOR(12*xpos,12*ypos,12,Image_Black,Image_Black,GetPlane(0),GetPlane(1));
				key = ngetchx();
				GraySprite16_XOR(12*xpos,12*ypos,12,Image_Black,Image_Black,GetPlane(0),GetPlane(1));
				
				
				
				if (key==KEY_LEFT)
				{
					xpos--;
					if (xpos<0)
					xpos = (width-1);
				}
				
				else if (key==KEY_RIGHT)
				{
					xpos++;
					if (xpos>(width-1))
					xpos = 0;
				}
				
				else if (key==KEY_UP)
				{
					ypos--;
					if (ypos<0)
					ypos = (height-1);
				}
				
				else if (key==KEY_DOWN)
				{
					ypos++;
					if (ypos>(height-1))
					ypos = 0;
				}
				
				else if ((key==KEY_ENTER || key==KEY_F1)
				&& Game->Tiles[xpos][ypos].State==TS_Hidden)
				{
					
					
					
					if (Game->Pair.Count==pairs_size)
					{
						TPoint Pos=Game->Pair.Pos[0];
						Game->Tiles[Pos.x][Pos.y].State = TS_Hidden;
						Game->Pair.Count--;
						
						for (a=0;a<pairs_size-1;a++)
						Game->Pair.Pos[a]=Game->Pair.Pos[a+1];
					}
					
					
					
					Game->Pair.Pos[Game->Pair.Count].x = xpos;
					Game->Pair.Pos[Game->Pair.Count].y = ypos;
					Game->Tiles[xpos][ypos].State = TS_Visible;
					
					Game->Pair.Count++;
					
					if (Game->Pair.Count==pairs_size)
					{
						
						// TEST FOR SAME CHARACTER
						
						int same_char = 1;
						for (b=1;b<pairs_size;b++)
						if (Game->Tiles[Game->Pair.Pos[0].x][Game->Pair.Pos[0].y].Value!=
						Game->Tiles[Game->Pair.Pos[b].x][Game->Pair.Pos[b].y].Value)
						same_char = 0;
						
						if (same_char)
						{
							for (a=0;a<pairs_size;a++)
							Game->Tiles[Game->Pair.Pos[a].x][Game->Pair.Pos[a].y].State = TS_Used;
							
							Game->Pair.Count = 0;
							
							pairs_counter--;
						}
					}
					
					
					RefreshScreen();
					
				}
				
				else if (key==KEY_ESC)
				{
					MenuNew2();
					MenuAddTitle("Exit Game?");
					MenuAddItem("Yes");
					MenuAddItem("No");
					
					GrayOff();
					MenuDo(ME_PAUSE);
					GrayOn();
					
					if (!Menu->ok || Menu->Index==2)
					{
						RefreshScreen();
					}
					else
					break;
				}
				
				else if (key==KEY_F2)
				{
					NewGame();
					RefreshScreen();
				}
				
				else if (key==KEY_APPS)
				{
					off();
				}
				
			}
			
			
			GrayOff();
			
			
			
		}
		else if (Menu->Index==2)
		{
			
			for (;;)
			{
				
				MenuNew2();
				MenuAddTitle("Options");
				sprintf(buffer,"Width : %d",width);
				MenuAddItem(buffer);
				sprintf(buffer,"Height : %d",height);
				MenuAddItem(buffer);
				sprintf(buffer,"Pairs Size : %d",pairs_size);
				MenuAddItem(buffer);
				
				MenuDo(ME_OPTIONS);
				
				if (!Menu->ok) break;
				
				if (Menu->Index==1)
				{
					if (width<8)
					width++;
					else
					width = 4;
				}
				else if (Menu->Index==2)
				{
					if (height<8)
					height++;
					else
					height = 4;
				}
				else if (Menu->Index==3)
				{
					if (pairs_size<4)
					pairs_size++;
					else
					pairs_size = 2;
				}
				
				
			}
			
		}
		else if (Menu->Index==3)
		{
			
			clrscr();
			FontSetSys(0);
			printf("About \n");
			printf("Memory 1.2 \n");
			printf("Programmed by Olivier Sangala \n");
			printf("http://olivier.sangala.free.fr \n");
			printf("olivier_sangala@hotmail.com \n");
			FontSetSys(1);
			ngetchx();
		}
		else if (Menu->Index==4)
		{
			break;
		}
		
	}
	
	
}
